/***********************************************************

    hbs reduction subroutine: hbsipol.c

                                   1998/09/28  K. Kawabata

    I, q, u, p and theta/calibrated magnitude ---> Total/Pol flux table

    usage: % hbsipol infn1 infn2 infn3 outfn

      infn1 -- input table made by hbsiqupt
      infn2 -- magnitude (FITS format)
      infn3 -- error ratio (FITS format)
      outfn -- file name of output table

    Output table contains
       column     contents
        1     wavelength (angstrom)
        2     magnutude (total flux)
        3     error( mag_tot )
        4     magnutude (unpolarized flux)
        5     error( mag_unp )
        6     magnutude (polarized flux)
        7     error( mag_pol )

 Magnitude can be converted into...

  fnu = 3.631e-20*10^{-0.4*mag} erg/sec/cm^2/Hz
     ( error(fnu) = 3.344e-20*10^{-0.4*mag}*error(mag) )

 or

  flmd = 0.10885/lmd^2 *10^{-0.4*mag} erg/sec/cm^2/A 
     ( error(fnu) = 0.10025/lmd^2 *10^{-0.4*mag}*error(mag) )

 where nu is Heltz unit and lmd is Angstrom unit.

************************************************************/

#include<stdio.h>
#include<string.h>
#include<math.h>
#include<stdlib.h>
#include"fitsio.h"

#include"hbsdefine.h"


int scanfile( char *infile, float count[]  ){

  fitsfile *fp;
  int i, anynul, status;
  char buf[200];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, infile, READONLY, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX3 || naxis2 != 1 ){
    printf( "error: definition of NX3 or NY(=1) in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_read_img( fp, TFLOAT, 1L, npix, &nulval,\
      count, &anynul, &status ) != 0 ){
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }

  return( 0 );

}

int main( int argc, char *argv[]){

  float Mag[NX3], Err[NX3];
  double P[NX3], Psig[NX3], Theta[NX3], Lmd[NX3];
  double Lamc[NX3], dLameff[NX3];
  char Lamw[NX3][15];
  double mag, err, p, psig, lmd;
  double q, qsig, u, usig, theta;
  double totmag, totmagsig;
  double unpmag, unpmagsig;
  double polmag, polmagsig;
  double a;
  double ONEDEG;
  int pix;
  FILE *fp;
  char buf[200], last_buf[200];
  char fn_mag[100], fn_err[100];
  char fn_intab[100], fn_outtab[100];
  int flag_mode;
  int npix;

  ONEDEG = 3.14159265358979 / 180.0;
 
  /***** confirmation of input file *****/

  if( argc != 5  ){
    printf( "usage: hbsipol intab mag err outtab\n" );
    exit( -1 );
  }

  strcpy( fn_intab, argv[1] );
  strcpy( fn_mag, argv[2] );
  strcpy( fn_err, argv[3] );
  strcpy( fn_outtab, argv[4] );

  /*** read magnitude and its error ***/

  if( scanfile( fn_mag, Mag ) != 0 ){
    printf( "Read error in '%s'\n", fn_mag );
    exit( -1 );
  }
  if( scanfile( fn_err, Err ) != 0 ){
    printf( "Read error in '%s'\n", fn_err );
    exit( -1 );
  }

  if( ( fp = fopen( fn_intab, "rt" ) ) == NULL ){
    printf( "cannot open %s", fn_intab );
    exit( -1 );
  }

  npix = 0;
  while( feof( fp ) == 0 ){
    fgets( buf, 200, fp );
    if( strcmp( last_buf, buf ) == 0 )
      continue;
    npix++;
    if( npix > NX3 ){
      printf( "Too many data in %s", fn_intab );
      exit( -1 );
    }
    pix = npix - 1;
    sscanf( buf, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %s %lf",\
	   &Lmd[pix],\
	   &a, &a, &a, &a, &a, &P[pix], &Psig[pix], &Theta[pix], &a,\
	   &Lamc[pix], Lamw[pix], &dLameff[pix] );
    strcpy( last_buf, buf );
  }
  if( npix == 0 ){
    printf( "No data in %s", fn_intab );
    exit( -1 );
  }

  fclose( fp );

  /*** open output file ***/

  if( ( fp = fopen( fn_outtab, "wt" ) ) == NULL ){
    printf( "cannot open %s", fn_outtab );
    exit( -1 );
  }

  for( pix=0; pix<=(npix-1); pix++ ){
      
    mag = Mag[pix];
    err = Err[pix];
    p = P[pix] / 100.0;
    psig = Psig[pix] / 100.0;
    theta = Theta[pix] * ONEDEG;
    lmd = Lmd[pix];

    totmag = mag;
    totmagsig = err * 1.086;
    unpmag = mag - 2.5 * log10( 1.0 - p );
    unpmagsig = 1.086 * sqrt( err * err + psig * psig );
    polmag = mag - 2.5 * log10( p );
    polmagsig = 1.086 * ( pow( 10.0, (-0.4*unpmag) ) * unpmagsig \
                / pow( 10.0, (-0.4*polmag ) ) );
    q = 0.10885 / ( lmd * lmd ) * pow( 10.0, ( -0.4 * polmag ) )\
        * cos( 2.0 * theta );
    u = 0.10885 / ( lmd * lmd ) * pow( 10.0, ( -0.4 * polmag ) )\
        * sin( 2.0 * theta );
    qsig = 0.10885 / ( lmd * lmd ) * pow( 10.0, ( -0.4 * polmag ) )\
        * polmagsig * 0.707107;
    usig = qsig;

    fprintf( fp, "%7.2f  %+7.3f %6.4f %+7.3f %6.4f %+7e %7e %+7e %7e %7.2f %s %7.2f\n",\
	    lmd, totmag, totmagsig, polmag, polmagsig, q, qsig, u, usig,\
	    Lamc[pix], Lamw[pix], dLameff[pix] );
  }
    
  fclose(fp);
    
  return( 0 );
}
